
import numpy
from scipy import *
from pylab import *

graphon = 1
from visual import *

def dx_dt_open(r,t=0):
    x[n+1:2*n-1] = (r[0:n-2] - 2*r[1:n-1] + r[2:n])
    x[n] = (r[1]-r[0])
    x[2*n-1] = (r[n-2]-r[n-1])
    x[0:n] = r[n:]
    return x

def energy_open(r):
    e = 0
    for i in range(0,n-1): 
        e += (r[i+1]-r[i])**2 + r[n+i]**2
    e += r[num_var-1]**2
    return 0.5*e

def f(y):
    return y
    #return y+0.5*y**3
    #return y+4.5*y**3

def dx_dt_fixed(r,t=0):
    delta = array([f(r[i+1]-r[i]) for i in range(n-1)])
    x[n+1:2*n-1] = delta[1:n-1]-delta[0:n-2]
    x[n] = f(r[1]-r[0])-f(r[0])
    x[2*n-1] = f(r[n-2]-r[n-1]) - f(r[n-1])
    x[0:n] = r[n:]
    return x

def energy_fixed(r):
    e = 0
    for i in range(0,n-1): 
        e += (r[i+1]-r[i])**2 + r[n+i]**2
    e += r[num_var-1]**2 + r[0]**2 + r[n-1]**2
    return 0.5*e

def modes_fixed(x):
    r = zeros(2*n+2,typecode='d')
    v = zeros(2*n+2,typecode='d')
    r[1:n+1] = x[0:n]
    v[1:n+1] = x[n:]
    for i in range (1,n+1): 
        r[2*n+2-i] = -r[i]
        v[2*n+2-i] = -v[i]
    ir, iv =  imag(fft(r)),imag(fft(v))
    outr = zeros(n,typecode='d')
    outv = zeros(n,typecode='d')
    for i in range(n):
        outr[i] = ir[i+1]
        outv[i] = iv[i+1]
    return outr, outv


def total_mode_energy(rq,vq):
    return sum(mode_energy(rq,vq))

def mode_energy(rq,vq):
    modes = zeros(n,typecode='d')
    for index in range(n):
        modes[index] = 0.5*(wq[index]*rq[index]**2+vq[index]**2)/(2*n+2)
    return modes

energy = energy_fixed
dx_dt = dx_dt_fixed
modes = modes_fixed

n=14
wq = [2*(1-cos(2*pi*(m+1)/(2*n+2))) for m in range(n)]
num_var = 2*n
x = zeros(num_var,typecode='d')
x0 = zeros(num_var,typecode='d')
#x0[:n] = standard_normal((n))
x0[:n] = array([sin(2*pi*(m+1)/(2*n+2)) for m in range(n)])
dt = 0.1
graph_step = 1.0
t = arange(0,graph_step,dt)
E=[]
mode_e = zeros(n,typecode='d')
mode_list1 = []
mode_list2 = []
mode_list = []

if graphon:
    floor = box (pos=(n,-4,-4), length=n*2, height=0.5, width=4, color=color.blue)
    spacing = 2.0
    ball = []
    for i in range(n): 
        ball.append(sphere (pos=(0,0,0), radius=1, color=color.red))

ion()
for i in range(10000): 
    if graphon: rate(100)
    x_out, infohash = integrate.odeint(dx_dt, x0, t, full_output=True)
    E.append(energy(x_out[-1,:]))
    x0 = x_out[-1]
    m = modes(x0)
    mode_e[0:n] = mode_energy(m[0],m[1])[0:n]
    mode_list.append(mode_e)
    mode_list1.append(mode_e[0])
    if (i%100 == 0):
        plot(mode_e,'g.')
        show()
        cla()

    #check that normal mode analysis gives same total energy
    print  total_mode_energy(m[0],m[1])/energy(x_out[-1])

    if graphon:
        for j in range(n):
         ball[j].pos = (x_out[-1,j]+spacing*j,0,0)


